#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include <inttypes.h>

#include"ili9341gfx.h"
#include"ili9341.h"
#include"resources.h"
#include"spi.h"


extern uint16_t vsetx,vsety,vactualx,vactualy,isetx,isety,iactualx,iactualy;

/* Main character position */
int16_t main_char_x;
int16_t main_char_y;

/* Dimensions of Flappy Bird */
#define FLAPPY_WIDTH (uint16_t) 33
#define FLAPPY_HEIGHT (uint16_t) 22

#define WALL_DIST (uint16_t) 60

//int to char functie pentru o singura cifra
char to_char(int x){
	switch(x){
		case 0:
			return '0';
			break;
	  case 1:
			return '1';
			break;
		case 2:
			return '2';
			break;
		case 3:
			return '3';
			break;
	  case 4:
			return '4';
			break;
		case 5:
			return '5';
			break;
		case 6:
			return '6';
			break;
		case 7:
			return '7';
			break;
		case 8:
			return '8';
			break;
		case 9:
			return '9';
			break;
	}
}


void init_btn(void) {
	/* Setez pinul PD0 ca intrare. */
	DDRD &= ~(1 << PD0);

	/* Activez rezistenta de pull-up pentru pinul PD0. */
	PORTD |= (1 << PD0);

	/* Setez pinul PD4 ca intrare. */
	DDRD &= ~(1 << PD4);

	/* Activez rezistenta de pull-up pentru pinul PD4. */
	PORTD |= (1 << PD4);
}


void start_game(void) {
	ili9341_init();//initial driver setup to drive ili9341

	int cont;
	int speed;
	uint16_t y;
	uint16_t wall_h1;
	uint16_t wall_x1;
	uint8_t score;
	//back-up variables
	int speed_bk = 0;
	uint16_t y_bk = 120;
	uint16_t wall_h1_bk = 100;
	uint16_t wall_x1_bk = 220;
	uint8_t score_bk = 0;

	uint16_t char_x=20;
	uint16_t char_y=12;
  //loop principal
	while (1)
	{
		ili9341_clear( BLUE); //fill screen with blue colour
		ili9341_fillrect(0,240,240,80,GREEN); //make the green ground
		cont = 0; // counts the frames
		speed = speed_bk; //the bird speed
		y = y_bk; // bird y position
		wall_h1 = wall_h1_bk; // the height of the wall (top-middle)
		wall_x1 = wall_x1_bk; // wall x position
		score = score_bk; // the score

		ili9341_fillrect(wall_x1,0,20,wall_h1,BLACK); // the obstacles top
		ili9341_fillrect(wall_x1,wall_h1+WALL_DIST,20,ILI9341_TFTHEIGHT-(wall_h1 + WALL_DIST)-80,BLACK); // the obstacles bottom
		char pd4 = 0; // checks the end game or the restart button

		//game loop
		while(pd4 == 0)
		{
			//afisarea scorului
			char_x = 10;
			char_y = 10;
			ili9341_drawchar(char_x, char_y, 'S',RED, BLUE, 1);
	 		char_x +=6;
			ili9341_drawchar(char_x, char_y, 'C',RED, BLUE, 1);
			char_x +=6;
			ili9341_drawchar(char_x, char_y, ':',RED, BLUE, 1);
			char_x +=6;
			y_bk = y;
			while(score / 10 != 0){
				ili9341_drawchar(char_x, char_y, to_char(score%10),RED, BLUE, 1);
				char_x +=6;
				score = score /10;
			}
			ili9341_drawchar(char_x, char_y, to_char(score%10),RED, BLUE, 1);

			//desenarea caracterului in functie de viteza(inclinare)
			//drept
			if (speed == 0) {
				drawBitmap(40, y, flappy_bird_map_s, FLAPPY_WIDTH, FLAPPY_HEIGHT, YELLOW, BLUE);
			}
			//inclinat dreapta
			if (speed < 0) {
				//calcularea pozitiei urmatoare si desenarea unui patrat in locul pozitiei curente
				if(cont % 4 ==0){
					ili9341_fillrect(40, y, 33, 22,BLUE);
					y -= speed*4;
				}
				//desenarea in pozitia noua
				else{
					drawBitmap(40, y, flappy_bird_map_r, FLAPPY_WIDTH, FLAPPY_HEIGHT, YELLOW, BLUE);
				}
			}
			if (speed > 0) {
				if(cont % 4 == 0){
				 ili9341_fillrect(40, y, 33, 22,BLUE);
					y -= speed*4;
				} else {
					drawBitmap(40, y, flappy_bird_map_l, FLAPPY_WIDTH, FLAPPY_HEIGHT, YELLOW, BLUE);
				}
			}

			//verificare buton jump
			if ((PIND & (1 << PD0)) == 0){
				speed += 3;
				if(speed > 5){
					speed = 5;
				}
			}
			//verificare buton de reset
			if ((PIND & (1 << PD4)) == 0){
				 pd4 = 1;
			}

			//incrementarea contorului
			cont += 1;
			//scaderea vitezei
			if(cont % 2 == 0){
				speed -= 1;
				if(speed < -4){
					speed = -4;
				}
			}
			//miscarea obstacolului
			ili9341_fillrect(wall_x1+16,0,4,wall_h1,BLUE);
			ili9341_fillrect(wall_x1+16,wall_h1+WALL_DIST,4,ILI9341_TFTHEIGHT-(wall_h1 + WALL_DIST)-80,BLUE);

			ili9341_fillrect(wall_x1-4,0,4,wall_h1,BLACK);
			ili9341_fillrect(wall_x1-4,wall_h1+WALL_DIST,4,ILI9341_TFTHEIGHT-(wall_h1 + WALL_DIST)-80,BLACK);

			wall_x1-=4;

			//verificarea ajungerii obstacolului la final
			if(wall_x1 == 0 ){
				score ++;
				ili9341_fillrect(wall_x1,0,20,wall_h1,BLACK);
				ili9341_fillrect(wall_x1,wall_h1+WALL_DIST,20,ILI9341_TFTHEIGHT-(wall_h1 + WALL_DIST)-80,BLACK);
				wall_x1 = 240;
				ili9341_fillrect(0,0,20,240,BLUE);

				//alegerea unei noi inaltimi pentru obstacol in functie de pozitia anterioara si viteza
				wall_h1 -= speed * 10;
				if(wall_h1 < 10){
					wall_h1 = 10;
				}
				if(wall_h1 > 180){
					wall_h1 = 180;
				}
			}
			//verificare coliziune cu obstacolul
			if(wall_x1 <= 40 + FLAPPY_WIDTH && wall_x1 >= 40 - 20 && (y_bk <= wall_h1 + 2|| y_bk >= wall_h1 + WALL_DIST - FLAPPY_HEIGHT -3 )){
				y_bk = y;
				ili9341_fillrect(40, y, 33, 22,BLUE);
				pd4 = 2;
			}

			//verificare coliziune cu exteriorul
			if(y >= 240-23){
				y_bk = y;
				y = 240-23;
				ili9341_fillrect(40, y, 33, 22,BLUE);
				pd4 = 2;
			}
		}
		//desenare text "RESET?" si scorul final pana in momentul respectiv
		cont = 0;
		char_x=20;
		char_y=120;
		ili9341_drawchar(char_x, char_y, 'R',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'E',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'S',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'T',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'A',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'R',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'T',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, '?',RED, BLUE, 4);
		char_y += 30;

		char_x=20;
		ili9341_drawchar(char_x, char_y, 'S',RED, BLUE, 4);
 		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'C',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'O',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'R',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, 'E',RED, BLUE, 4);
 		char_x +=22;
		ili9341_drawchar(char_x, char_y, ' ',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, '=',RED, BLUE, 4);
		char_x +=22;
		ili9341_drawchar(char_x, char_y, ' ',RED, BLUE, 4);
		char_x +=22;
		while(score / 10 != 0){
			ili9341_drawchar(char_x, char_y, to_char(score%10),RED, BLUE, 4);
			char_x +=22;
			score = score /10;
		}
		ili9341_drawchar(char_x, char_y, to_char(score%10),RED, BLUE, 4);


		while(pd4 == 1){
			//verificare butoane (resetarea jocului cu date noi sau cu datele anterioare)
			if(cont == 10000){
				if ((PIND & (1 << PD0)) == 0){
					 pd4 = 0;
					 y_bk = y;
					 speed_bk = speed;
					 wall_h1_bk = wall_h1;
					 wall_x1_bk = wall_x1;
				}
				if ((PIND & (1 << PD4)) == 0){
					 pd4 = 0;
					 y_bk = 120;
					 speed_bk = 0;
					 wall_h1_bk = 100;
					 wall_x1_bk = 220;
				}
			}
			cont++;
		}


		while(pd4 == 2){
			//desenare dead flappy bird
			if(wall_x1 <= 40 + FLAPPY_WIDTH){
				drawBitmap(wall_x1-FLAPPY_WIDTH, 240-22, flappy_bird_map_dead, FLAPPY_WIDTH, FLAPPY_HEIGHT, YELLOW, BLUE);

			}else{
				drawBitmap(40, 240-22, flappy_bird_map_dead, FLAPPY_WIDTH, FLAPPY_HEIGHT, YELLOW, BLUE);
			}

			//verificare butoane pentru reinceperea jocului
			if ((PIND & (1 << PD4)) == 0){
				 pd4 = 0;
				 y_bk = 120;
				 speed_bk = 0;
				 wall_h1_bk = 100;
				 wall_x1_bk = 220;
			}
			if ((PIND & (1 << PD0)) == 0){
				 pd4 = 0;
				 y_bk = 120;
				 speed_bk = 0;
				 wall_h1_bk = 100;
				 wall_x1_bk = 220;
			}

		}
	}
}


int main(void)
{
    init_btn();
    start_game();
}
